@@ -13,16 +13,45 @@ import vcs
 from vcs import git, hg
 manifest = None
 import metadata
+import testloader
 import wptcommandline
 
 base_path = os.path.abspath(os.path.split(__file__)[0])
 
+bsd_license = """W3C 3-clause BSD License
 
-def do_test_relative_imports(test_root):
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are
+met:
+
+* Redistributions of works must retain the original copyright notice, this
+  list of conditions and the following disclaimer.
+
+* Redistributions in binary form must reproduce the original copyright
+  notice, this list of conditions and the following disclaimer in the
+  documentation and/or other materials provided with the distribution.
+
+* Neither the name of the W3C nor the names of its contributors may be
+  used to endorse or promote products derived from this work without
+  specific prior written permission.
+
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
+"""
+
+def do_delayed_imports(serve_root):
     global manifest
-
-    sys.path.insert(0, os.path.join(test_root))
-    sys.path.insert(0, os.path.join(test_root, "tools", "scripts"))
+    sys.path.insert(0, os.path.join(serve_root, "tools", "scripts"))
     import manifest
 
 
@@ -119,6 +148,12 @@ class WebPlatformTests(object):
             dest_path = os.path.join(dest, destination, os.path.split(source)[1])
             shutil.copy2(source_path, dest_path)
 
+        self.add_license(dest)
+
+    def add_license(self, dest):
+        with open(os.path.join(dest, "LICENSE"), "w") as f:
+            f.write(bsd_license)
+
 
 class NoVCSTree(object):
     name = "non-vcs"
@@ -179,10 +214,17 @@ class HgTree(object):
         try:
             self.hg("qinit")
         except subprocess.CalledProcessError:
-            # There is already a patch queue in this repo
-            # Should only happen during development
             pass
-        self.hg("qnew", patch_name, "-X", self.root, "-m", message)
+
+        patch_names = [item.strip() for item in self.hg("qseries").split("\n") if item.strip()]
+
+        suffix = 0
+        test_name = patch_name
+        while test_name in patch_names:
+            suffix += 1
+            test_name = "%s-%i" % (patch_name, suffix)
+
+        self.hg("qnew", test_name, "-X", self.root, "-m", message)
 
     def update_patch(self, include=None):
         if include is not None:
@@ -227,9 +269,8 @@ class GitTree(object):
         self.git("add", *args)
 
     def create_patch(self, patch_name, message):
-        # In git a patch is actually a branch
+        # In git a patch is actually a commit
         self.message = message
-        self.git("checkout", "-b", patch_name)
 
     def update_patch(self, include=None):
         assert self.message is not None
@@ -269,16 +310,25 @@ def ensure_exists(path):
 def sync_tests(paths, local_tree, wpt, bug):
     wpt.update()
 
+    do_delayed_imports(paths["sync"])
+
     try:
         #bug.comment("Updating to %s" % wpt.rev)
-        initial_manifest = metadata.load_test_manifest(paths["sync"], paths["metadata"])
-        wpt.copy_work_tree(paths["test"])
-        new_manifest = metadata.update_manifest(paths["sync"], paths["metadata"])
+        sync_paths = {"/": {"tests_path": paths["sync"],
+                            "metadata_path": paths["sync_dest"]["metadata_path"]}}
+
+        manifest_loader = testloader.ManifestLoader(sync_paths)
+        initial_manifest = manifest_loader.load()
+        wpt.copy_work_tree(paths["sync_dest"]["test_path"])
+        new_manifest = metadata.update_manifest(paths["sync"],
+                                                paths["sync_dest"]["metadata_path"])
 
         local_tree.create_patch("web-platform-tests_update_%s" % wpt.rev,
                                 "Update web-platform-tests to revision %s" % wpt.rev)
-        local_tree.add_new(os.path.relpath(paths["test"], local_tree.root))
-        local_tree.update_patch(include=[paths["test"], paths["metadata"]])
+        local_tree.add_new(os.path.relpath(paths["sync_dest"]["test_path"],
+                                           local_tree.root))
+        local_tree.update_patch(include=[paths["sync_dest"]["test_path"],
+                                         paths["sync_dest"]["metadata_path"]])
     except Exception as e:
         #bug.comment("Update failed with error:\n %s" % traceback.format_exc())
         sys.stderr.write(traceback.format_exc())
@@ -289,20 +339,27 @@ def sync_tests(paths, local_tree, wpt, bug):
     return initial_manifest, new_manifest
 
 
-def update_metadata(paths, local_tree, wpt, initial_rev, bug, log_files, ignore_existing):
+def update_metadata(paths, local_tree, initial_rev, bug, log_files, ignore_existing,
+                    wpt_repo=None):
     try:
         try:
-            local_tree.create_patch("web-platform-tests_update_%s_metadata" % wpt.rev,
-                                    "Update web-platform-tests expected data to revision %s" %
-                                    wpt.rev)
+            if wpt_repo is not None:
+                name = "web-platform-tests_update_%s_metadata" % wpt_repo.rev
+                message = "Update web-platform-tests expected data to revision %s" % wpt_repo.rev
+            else:
+                name = "web-platform-tests_update_metadata"
+                message = "Update web-platform-tests expected data"
+
+            local_tree.create_patch(name, message)
         except subprocess.CalledProcessError:
             # Patch with that name already exists, probably
             pass
-        needs_human = metadata.update_expected(paths["sync"],
-                                               paths["metadata"],
+        needs_human = metadata.update_expected(paths["test_paths"],
+                                               paths["serve"],
                                                log_files,
                                                rev_old=initial_rev,
-                                               ignore_existing=ignore_existing)
+                                               ignore_existing=ignore_existing,
+                                               sync_root=paths.get("sync", None))
 
         if needs_human:
             #TODO: List all the files that should be checked carefully for changes.
@@ -321,12 +378,18 @@ def update_metadata(paths, local_tree, wpt, initial_rev, bug, log_files, ignore_
 def run_update(**kwargs):
     config = kwargs["config"]
 
-    paths = {"sync": kwargs["sync_path"],
-             "test": kwargs["tests_root"],
-             "metadata": kwargs["metadata_root"]}
+    paths = {}
+    paths["test_paths"] = kwargs["test_paths"]
 
-    for path in paths.itervalues():
-        ensure_exists(path)
+    if kwargs["sync"]:
+        paths["sync_dest"] = kwargs["test_paths"]["/"]
+        paths["sync"] = kwargs["sync_path"]
+
+
+    paths["serve"] = kwargs["serve_root"] if kwargs["serve_root"] else paths["test_paths"]["/"]["tests_path"]
+
+#    for path in paths.itervalues():
+#        ensure_exists(path)
 
     if not kwargs["sync"] and not kwargs["run_log"]:
         print """Nothing to do.
@@ -352,19 +415,26 @@ expected data."""
     if rev is None:
         rev = config["web-platform-tests"].get("branch", "master")
 
-    wpt = WebPlatformTests(config["web-platform-tests"]["remote_url"],
-                           paths["sync"],
-                           rev=rev)
     bug = None
 
     initial_rev = None
+    wpt_repo = None
+
     if kwargs["sync"]:
-        initial_manifest, new_manifest = sync_tests(paths, local_tree, wpt, bug)
+        wpt_repo = WebPlatformTests(config["web-platform-tests"]["remote_url"],
+                                    paths["sync"],
+                                    rev=rev)
+        initial_manifest, new_manifest = sync_tests(paths, local_tree, wpt_repo, bug)
         initial_rev = initial_manifest.rev
 
     if kwargs["run_log"]:
-        update_metadata(paths, local_tree, wpt, initial_rev, bug,
-                        kwargs["run_log"], kwargs["ignore_existing"])
+        update_metadata(paths,
+                        local_tree,
+                        initial_rev,
+                        bug,
+                        kwargs["run_log"],
+                        kwargs["ignore_existing"],
+                        wpt_repo=wpt_repo)
 
 
 def main():
